program Circles;
(****************************************************************************
** Demonstration of drawing solid circles on the VGA screen                **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************)

uses Crt;

type BufferType = Array [0..63999] of Byte;

var
  {The screen itself}
  Screen  : BufferType absolute $A000:0000;
  {An offscreen buffer}
  Buffer  : ^BufferType;

{Switches to VGA 320x200 256 colour}
procedure VGAScreen; assembler;
asm
  mov ax, $13
  int $10
end;

{Switches to text 80x25}
procedure TextScreen; assembler;
asm
  mov ax, $3
  int $10
end;

{Sets a pixel in the buffer to a specific colour}
procedure PutPixel (x, y : Word; c : Byte);
begin
  Buffer^ [y shl 8 + y shl 6 + x] := c;
end;

{Draws a horizontal line from (x1,y) to (x2,y)}
procedure DrawHorLine (x1, y, x2 : Integer; c : Byte);
var
  offset, length : Integer;

begin
  {Clip to vertical screen size}
  if (y < 0) or (y > 200) then exit;

  {Swap endpoints if necessary}
  if (x1 > x2) then begin
    length := x2;
    x2 := x1;
    x1 := length;
  end;

  {Clip to horizontal screen size}
  if (x2 < 0) or (x1 > 319) then exit;
  if (x1 < 0) then x1 := 0;
  if (x2 > 319) then x2 := 319;

  {Calculate address and length}
  offset := (y shl 8) + (y shl 6) + x1;
  length := x2 - x1 + 1;

  {Draw the line}
  FillChar (Buffer^ [offset], length, c);
end;

{Draws a filled circle with the specified centerpoint and radius}
procedure DrawSolidCircle (centerx, centery, radius : Integer; colour : Byte);
var
  line, position, x, y : Integer;
  sqrad : LongInt;

begin
  {No radius is a single pixel}
  if (radius < 1) then begin
    PutPixel (centerx, centery, colour);
    exit;
  end;

  {Square radius}
  sqrad := radius * radius;

  {Center line}
  DrawHorLine (centerx - radius, centery, centerx + radius, colour);

  {Rest of the circle}
  dec (Radius);
  for line := 1 to radius do begin
    {Calculate position}
    position := trunc (sqrt (sqrad - (line * line)));

    {Draw horizontal lines, taking advantage of circle's perfect symmetry}
    DrawHorLine (centerx - position, centery - line, centerx + position, colour);
    DrawHorLine (centerx - position, centery + line, centerx + position, colour);
  end;
end;
var
  CenterX, CenterY, Radius, Colour : Integer;

begin

  {Initialize graphics}
  VGAScreen;

  {I'm drawing to the screen instead of a buffer}
  Buffer := @Screen;

  {Keep drawing circles until a keypress}
  repeat
    CenterX := random (320);
    CenterY := random (200);
    Radius := random (50);
    Colour := random (256);
    DrawSolidCircle (CenterX, CenterY, Radius, Colour);
  until KeyPressed;

  {Back to text}
  TextScreen;
end.